"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const glob_1 = tslib_1.__importDefault(require("glob"));
const path_1 = tslib_1.__importDefault(require("path"));
const fs_1 = tslib_1.__importDefault(require("fs"));
/**
 * combine d.ts files => index.d.ts as an entry point file
 *
 * @export
 */
function combine(options = {}) {
    const tsRoot = options.tsRoot || process.cwd();
    const files = glob_1.default
        .sync('./**/*.ts', { cwd: tsRoot })
        .map((d) => path_1.default.resolve(tsRoot, d));
    const isFilenameConflict = !!files.find((f) => f === options.entryName);
    if (isFilenameConflict) {
        console.log(`\u001b[33mentry filename [${options.entryName}] is the same with output d.ts filename, please rename it.\u001b[39m`);
        return;
    }
    const lines = files.map((f) => {
        const relativePath = path_1.default.relative(tsRoot, f);
        if (options.useModule)
            return `export * from "./${relativePath.replace(/\.d\.ts$/, '')}";`;
        else
            return `/// <reference path="${relativePath}" />`;
    });
    fs_1.default.writeFileSync(options.entryName || 'index.d.ts', lines.join('\n') + '\n', 'utf8');
}
exports.default = combine;
